/*
 *  remailck.c
 *
 *  An RFC 1339 remote mailcheck client
 *
 *  Copyright (C) 2002 Tim Deegan (tjd@phlegethon.org)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  $Id: remailck.c,v 1.7 2002/06/14 14:29:45 tjd Rel $
 *
 */


#include <errno.h>
#include <netdb.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define TIMEOUT 15                 /* Number of seconds to wait for a reply  */
#define REMAILCK_PORT 50           /* Port the server should be listening on */
#define UDP_PAYLOAD (65535 - 28)   /* IP datagram - IP header - UDP header   */

/* Globals */
char *me;

/* Helpers */
static void timeout(int sig) 
{
    fprintf(stderr, "%s: timed out waiting for reply.\n", me);
    exit(-1);
}

/* Master control */
int main(int argc, char **argv)
{
    char *host, *req;
    struct hostent *he;
    struct sockaddr_in sockaddr;
    socklen_t addrlen;
    ssize_t reqlen;
    int s, i;
    uint32_t old, new, reply[3];
    time_t now, old_t, new_t;

    /* Learn my name */
    me = strrchr(argv[0], '/');
    if (me == NULL) me = argv[0]; else me++;

    /* Check arguments */
    if (argc != 2
	|| (host = strchr(argv[1], '@')) == NULL
	|| host == argv[1]
	|| host[1] == '\0')
    {
 	fprintf(stderr, "usage: %s user@hostname\n", me);
 	exit(-1); 
    }
    
    /* Separate username from host */
    *host++ = '\0';
    if (strlen(argv[1]) + 5 > UDP_PAYLOAD) {
	fprintf(stderr, "%s username (far) too long\n", me);
	exit(-1);
    }

    /* Translate the hostname */
    if ((he = gethostbyname(host)) == NULL)  
    {
	fprintf(stderr, "%s: %s\n", me, hstrerror(h_errno));
	exit(-1);
    }
    if (he->h_addrtype != AF_INET || he->h_length != 4)
    {
	fprintf(stderr, "%s: \"%s\" has a non-AF_INET address.\n", me, host);
	exit(-1);
    }
    memset(&sockaddr, 0, sizeof(sockaddr));
    sockaddr.sin_family = AF_INET;
    sockaddr.sin_port = htons(REMAILCK_PORT);
    memcpy(&sockaddr.sin_addr, he->h_addr, 4);
    addrlen = sizeof(sockaddr);

    /* Prepare a socket */
    s = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (s < 0) {
	fprintf(stderr, "%s: can't open a socket.\n", me, host);
	exit(-1);
    }

    /* Prepare a request: 4 null bytes and an ASCII username */
    reqlen = strlen(argv[1]) + 5;
    if ((req = malloc(reqlen)) == NULL) {
	fprintf(stderr, "%s: can't malloc() for request.\n", me);
	exit(-1);
    }
    req[0] = req[1] = req[2] = req[3] = '\0';
    strcpy(req+4, argv[1]);
    
    /* Fetchez! */
    if (signal(SIGALRM, timeout) == SIG_ERR) {
	fprintf(stderr, "%s: can't set timeout handler.\n", me);
	exit(-1);
    }
    if ((i = sendto(s, req, reqlen, 0, (struct sockaddr *)&sockaddr, addrlen))
	!= reqlen) 
    {
	fprintf(stderr, "%s: %s\n", me, (i==-1)?strerror(errno):"short tx");
	exit(-1);
    }
    alarm(TIMEOUT);
    if ((i = recvfrom(s, reply, 12, 0, (struct sockaddr *)&sockaddr, &addrlen))
	!= 12)
    {
	fprintf(stderr, "%s: %s\n", me, (i!=-1)?"short rx":strerror(errno));
	exit(-1);
    } 
    
    /* Decode */
    if (reply[0] != 0) {
	fprintf(stderr, "%s: server requested authentication\n", me);
	exit(-1);
    }
    now = time(NULL);
    new = ntohl(reply[1]);
    old = ntohl(reply[2]);

    if (old == 0 && new == 0) {
	printf("No mail.\n");
	return 2;
    } else if (old == 0 && new == 1) {
	printf("New mail.\n");
	return 0;
    } else if (old == 1 && new == 0) {
	printf("No new mail.\n");
	return 1;
    } else {
	old_t = now - old;
	printf("Mail last read: %s", ctime(&old_t));
	if (new < old) {
	    new_t = now - new;
	    printf("Most recent mail: %s", ctime(&new_t));
	    return 0;
	} else {
	    printf("No new mail.\n");
	    return 1;
	}
    }
}
